#
# spec file for package c++-gtk-utils
#
# Copyright (c) 2021 SUSE LLC
# Copyright (c) 2012 Malcolm J Lewis <malcolmlewis@opensuse.org>
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#


%global flavor gtk3%{nil}
%global soname 2_2-0

%global srcname c++-gtk-utils

%if "%{flavor}" == ""
ExclusiveArch:  do_not_build
%define pkgname %{srcname}
%endif

%if "%{flavor}" == "gtk2"
%bcond_without gtk
%define _gtk 2
%define config_script configure-gtk2
%define gtk_dev_pkg pkgconfig(gtk+-2.0)
%define pkgname %{srcname}-gtk%{_gtk}
%endif

%if "%{flavor}" == "gtk3"
%bcond_without gtk
%define _gtk 3
%define config_script configure
%define gtk_dev_pkg pkgconfig(gtk+-3.0)
%define pkgname %{srcname}-gtk%{_gtk}
%endif

%if "%{flavor}" == "gtk4"
%bcond_without gtk
%define _gtk 4
%define config_script configure-gtk4
%define gtk_dev_pkg pkgconfig(gtk4)
%define pkgname %{srcname}-gtk%{_gtk}
%endif

%define libname libcxx-gtk-utils-%{_gtk}-%{soname}
%define devname libcxx-gtk-utils-%{_gtk}-devel

Name:           %{pkgname}
Version:        2.2.20
Release:        1.15
Summary:        Lightweight library for GTK+ programs using C++
License:        LGPL-2.1-only
Group:          System/Libraries
URL:            http://cxx-gtk-utils.sourceforge.net/
Source0:        http://downloads.sourceforge.net/project/cxx-gtk-utils/cxx-gtk-utils/%{version}/%{srcname}-%{version}.tar.gz
BuildRequires:  autoconf
BuildRequires:  gcc-c++
BuildRequires:  guile-devel
%if %{with gtk}
BuildRequires:  %{gtk_dev_pkg}
%endif

%description
This is a lightweight library containing a number of classes and
functions for programming GTK+ programs using C++ in POSIX (unix-like)
environments, where the user does not want to use a full-on wrapper
such as gtkmm or wxWidgets.

%package -n %{libname}
Summary:        Lightweight library for GTK+ programs using C++
Group:          System/Libraries

%description -n %{libname}
This is a lightweight library containing a number of classes and
functions for programming GTK+ programs using C++ in POSIX (unix-like)
environments, where the user does not want to use a full-on wrapper
such as gtkmm or wxWidgets.

%package -n %{devname}
Summary:        Lightweight library for GTK+ programs using C++ -- Development Files
Group:          Development/Libraries/C and C++
Requires:       %{libname} = %{version}

%description -n %{devname}
This is a lightweight library containing a number of classes and
functions for programming GTK+ programs using C++ in POSIX (unix-like)
environments, where the user does not want to use a full-on wrapper
such as gtkmm or wxWidgets.

%prep
%autosetup -n %{srcname}-%{version} -p1

%build
if [ "%{config_script}" != "configure" ]
then
  cp -a %{config_script} configure
  cp -a %{config_script}.ac configure.ac
fi

%configure \
    --disable-static \
    --docdir=%{_docdir}/%{pkgname} \
    %{?with_gtk:--with-gtk} \
    %{!?with_gtk:--without-gtk} \
    %{nil}
%make_build

%install
%make_install
# Clean up
find %{buildroot} -type f -name "*.la" -delete -print

%post -n %{libname} -p /sbin/ldconfig
%postun -n %{libname} -p /sbin/ldconfig

%files -n %{libname}
%license COPYING
%doc NEWS
%{_libdir}/libcxx-gtk-utils-%{_gtk}-2.2.so.*

%files -n %{devname}
%doc ChangeLog
%doc %{_docdir}/%{pkgname}
%{_includedir}/c++-gtk-utils-%{_gtk}-2.2/
%{_libdir}/libcxx-gtk-utils-%{_gtk}-2.2.so
%{_libdir}/pkgconfig/c++-gtk-utils-%{_gtk}-2.2.pc

%changelog
* Tue Jan  5 2021 Atri Bhattacharya <badshah400@gmail.com>
- Update to version 2.2.20:
  + If this library is employed in a program compiled with
  - std=c++17 or greater, permit objects with references or const
    members to be stored in AsyncChannel objects
    (async_channel.h).
  + Deal with Cgu::WinBase and Cgu::Application breakage when
    testing using glib for gtk-3.99.3 onwards.
  + When compiling for GTK-4, require version >= 3.98.4 and make
    consequential changes including to documentation
    (acinclude.m4, README; doxygen.main.in).
  + Add fixes for gtk-3.98.0, gtk-3.98.1 and gtk-3.98.4
    (file_print_manager.cpp, text_print_manager.cpp, window.h,
    window.cpp; tests/test_application.cpp;
    tests/extra/test_file_print_manager.cpp,
    tests/extra/test_text_print_manager.cpp).
  + Enable Cgu::FilePrintManager dialogs to be resizable
    (file_print_manager.cpp).
  + Update documentation (application.h, parallel.h,
    shared_handle.h, window.h).
- Drop c++-gtk-utils-gtk4-3.98-fixes.patch: incorporated upstream.
* Wed Apr 29 2020 Atri Bhattacharya <badshah400@gmail.com>
- Update to version 2.2.19:
  + Use autoconf's TAP support for running tests.
  + Cater for guile-3.0 declarative modules.
  + Suppress GStaticRecMutex deprecation warnings.
- Changes from versions 2.2.16 through 2.2.18: See installed
  ChangeLog file.
- Convert to multibuild format to build flavours for gtk2, gtk3,
  and gtk4.
- Add BuildRequires: autoconf which is needed when building gtk2
  and gtk4 flavours where the configure script is modified before
  running %%configure.
- Add c++-gtk-utils-gtk4-3.98-fixes.patch to fix compilation
  against gtk4 >= 3.98; combined from upstream git master commits
  3589475, 687abc, 847e9b, and 97eb26 and rebased to apply cleanly
  using quilt.
* Sun Jan  7 2018 luc14n0@linuxmail.org
- Update to version 2.2.15:
  + Use streambuffers not streams when testing fdstream and gstream
    with char16_t and char32_t character types.
  + Update documentation.
- Pass --docdir to configure to correct the documentation's
  destination folder.
* Fri Nov 25 2016 zaitor@opensuse.org
- Update to version 2.2.14.1:
  + Explicitly qualify malloc() as being within the std namespace.
  + Do not document the Cgu::Thread::FutureHelper and
    Cgu::Thread::TaskManagerHelper2 namespaces.
  + Update documentation.
* Fri Nov 25 2016 zaitor@opensuse.org
- Update to version 2.2.14:
  + Provide Cgu::AsyncChannel.
  + Remove the 'noexcept' specifier for Cgu::Thread::Cond::wait()
    and Cgu::Thread::Cond::timed_wait(): any cancellation
    pseudo-exception is inconsistent with this specifier.
  + Provide TaskManager::get_max_tasks() method with documentation
    on rate limiting.
  + Zero initialise the character buffer in Reassembler class.
  + Adjust configuration macros to enable use of Cgu::Extension
    with guile-2.2.
  + Increase coverage of io_watch tests.
  + Update documentation.
* Sun May 15 2016 zaitor@opensuse.org
- Update to version 2.2.13:
  + Use glib's unix fd implementation for io_watch where glib
    >= 2.36 is installed, unless otherwise chosen during
    configuration.
  + Update documentation.
* Sun Nov  1 2015 p.drouand@gmail.com
- Update to version 2.2.12
  + Simplify Cgu::Application::add() method.
  + Minor implementation improvement for
    Cgu::Thread::parallel_transform* functions.
  + Permit return value optimisation when returning thread object
    on starting new thread.
  + Update libtool to version 2.4.6.
  + Documentation fix for doxygen-1.8.9.
  + Update code comments.
  + Update documentation.
* Tue Apr  7 2015 zaitor@opensuse.org
- Update to version 2.2.11:
  + Provide a mutex for calls to make-fresh-user-modules to deal
    with guile thread safety issues.
  + Improve efficiency of scheme to C++ list translators for long
    lists.
  + Update libtool to version 2.4.4.
  + Update code comments.
  + Update documentation.
* Fri Dec  5 2014 dimstar@opensuse.org
- Update to version 2.2.10:
  + Permit Cgu::Thread::parallel_for_each(),
    Cgu::Thread::parallel_for_each_partial(),
    Cgu::Thread::parallel_transform() and
    Cgu::Thread::parallel_transform_partial() to take const source
    iterators.
  + Provide Cgu::IntIter class to enable for loops to be more
    easily implemented with the parallel algorithms.
  + With gcc >= 4.8 or clang >= 3.4, implement callback classes
    using std::tuple (this enables Callback::make() and
    Callback::make_ref() to take up to 10 bound arguments, avoids
    a lot of code duplication and makes for a cleaner
    implementation).
  + Preallocate vectors in translator functions which return
    std::vector.
  + Correct information about clang compatibility.
  + Update documentation.
* Thu Oct 16 2014 zaitor@opensuse.org
- Update to version 2.2.9:
  + Provide AsyncQueue::move_pop_basic(),
    AsyncQueueDispatch::move_pop_basic(),
    AsyncQueueDispatch::move_pop_dispatch_basic() and
    AsyncQueueDispatch::move_pop_timed_dispatch_basic() methods.
  + Prevent new GTK+-3.14 deprecation warnings interfering with
    tests.
  + Update build system to automake-1.14.1.
  + Update build system to gettext-0.18.2.
  + Update tests.
  + Add comments about strict aliasing with character set
    conversion functions.
  + Update documentation, including to cover C++14.
* Sat Aug  9 2014 dimstar@opensuse.org
- Update to version 2.2.8:
  + Correct testing of minimum values in
    Extension::real_to_double() and
    Extension::list_to_vector_double().
  + Deal properly with unexpected out-of-memory conditions in
    executing Extension::exec() and Extension::exec_shared().
  + Minor code refactoring in EmitterArg::connect() and
    SafeEmitterArg::connect().
  + Update documentation.
- Changes from version 2.2.7:
  + Provide a new Extension::exec_shared() function.
  + Throw a C++ exception if a translator throws a native guile
    exception, simplify exception handling code and make the
    translation of guile exceptions to text more meaningful.
  + Block asyncs when executing a scheme to C++ translator where
    guile supports it.
  + Avoid constructing a SafeEmitterArg object with untracked
    io_watches.
  + Deal internally with any exceptions in
    cgu_winbase_delete_event().
  + Update documentation.
- Changes from version 2.2.6:
  + Provide Cgu::Callback::to_unique(), Cgu::Callback::to_functor()
    and Cgu::Callback::to_safe_functor() functions.
  + Make minor adjustments to handling of C++ exceptions in
    Cgu::Extension::exec().
  + Fix Cgu::Application tests for API breakage in GTK+-3.12.0.
  + Update documentation.
* Sun Feb  2 2014 zaitor@opensuse.org
- Update to version 2.2.5.1:
  + Deal correctly with a thrown guile exception which has a key
    but no other arguments, and a scheme extension file which does
    not throw but evaluates to a null list (extension.h).
  + Adjust timing of TaskManager releaser tests
    (tests/test_task_manager.cpp).
- Changes from version 2.2.5:
  + Provide support for guile extensions.
  + TaskManager::make_task_packaged_compose() methods, which will
    pass a std::future object for a task running on a TaskManager
    object asynchronously (task_manager.h, task_manager.tpp;
    tests/test_task_manager.cpp).
  + Minor code refactoring and test improvements (parallel.h,
    task_manager.tpp; tests/test_task_manager.cpp).
  + Update code comments (async_queue.h, task_manager.h,
    task_manager.tpp).
  + Update documentation (task_manager.h).
- Add guile-devel BuildRequires to build the new guile extensions.
* Sat Dec  7 2013 zaitor@opensuse.org
- Update to version 2.2.4:
  + Provide a TaskManager::make_task_packaged() method, which will
    return a std::future object for a task running on a TaskManager
    object.
  + Provide the expected result where TaskManager::add_task() is
    called concurrently with TaskManager::stop_all() in
    wait_for_running stop mode.
  + Guarantee strong exception safety of
    AsyncQueue::push(value_type&&) and (task_manager.cpp).
  + Guarantee strong exception safety of
    AsyncQueue::push(value_type&&) and
    AsyncQueueDispatch::push(value_type&&) for std::list container
    specialization.
  + Stop deprecation warnings with GTK+-3.10 in tests.
  + Updated documentation.
* Sat Sep 21 2013 zaitor@opensuse.org
- Update to version 2.2.3:
  + Revise implementation of parallel_transform() functions and add
    parallel_for_each_partial() and parallel_transform_partial()
    functions returning position iterators.
  + Provide a specialization of AsyncQueue and AsyncQueueDispatch
    for std::list containers which has push() and emplace() members
    which do not allocate within the mutex.
  + Improve scalability of TaskManager objects by starting new
    threads and adding to the task queue outside the mutex.
  + As the 2.2 series of c++-gtk-utils has minimum requirements of
    gcc >= 4.6 and clang >= 3.2, make non-throwing functions
    >'noexcept' where this does not affect ABI.
  + Really delete copy constructor and assignment operator of
    Thread::RWLock::WriterTrackLock.
  + Update documentation.
* Sun Aug 18 2013 zaitor@opensuse.org
- Update to version 2.2.2:
  + Add parallel_for_each() and parallel_transform() functions for
    TaskManager objects.
  + Provide a move constructor and move assignment operator for
    MainWidgetBase, PipeFifo and ScopedHandle classes.
  + Really delete fdstream's copy constructor and assignment
    operator.
  + Do not deprecate the version of Future::make() taking a
    function pointer and no arguments - it is equivalent to a
    callable object.
  + Add documentation on using the library with Qt.
  + Update documentation.
* Thu Jul  4 2013 zaitor@opensuse.org
- Update to version 2.2.1:
  + Provide a Thread::TaskManager::IncHandle class, and deprecate
    the use of TaskManager::set_max_threads.
  + Make the TaskManager::add_task() overload taking callable
    objects fully exception safe.
  + Issue a g_critical warning if a Thread::TaskManger task expires
    with an exception, there is no 'fail' callback and the thrown
    object is not Thread::Exit.
  + Replace AC_CONFIG_HEADER macro with AC_CONFIG_HEADERS when
    configuring.
  + Update build system to automake-1.13.3.
  + Upgrade doxygen configuration files to doxygen-1.8.4.
  + Update documentation.
* Sat Jun 22 2013 zaitor@opensuse.org
- Update to version 2.2.0:
  + Un-deprecate the overloads of EmitterArg::connect() taking a
    Callback::FunctorArg object. This was not intended.
  + Update git links following sourceforge repository changes.
  + Update unit tests.
  + Update documentation.
* Thu May 23 2013 dimstar@opensuse.org
- Update to version 2.1.0:
  + Bump version to version 2.1.0 in the 2.2 series of
    c++-gtk-utils.
  + Provide a generic callable object interface for
    Callback::post(), start_iowatch(), start_timeout(),
    Emitter::connect(), Future::fail(), Future::when(),
    Notifer::connect(), SafeEmitter::connect(),
    TaskManager::add_task(), TaskManager::make_task_when(),
    TaskManager::make_task_when_full(),
    TaskManager::make_task_compose() and Thread::start();
    deprecate the starting of tasks using Thread::Future and
    Thread::TaskManager except by callable objects.
  + Make move construction of AsyncQueue and AsyncQueueDispatch
    objects strongly exception safe (async_queue.h).
  + Remove the --with-auto-ptr option.
  + Simplify emitter/releaser implementation now lambdas and range-for
    are available in gcc-4.6 and above (emitter.h, emitter.cpp).
  + No longer document the deprecated Callback::make_val() functions.
  + Remove redundant Callback::Callback_function class.
  + Update build system to automake-1.13.1.
  + Stop clang++ mismatched-tags and unused-private-field warnings.
  + Cause bootstrap scripts to build translation files.
  + Make other documentation changes.
- Change soname 2_0-0 defines to 2_2-0: this results in a renamed
  package, according to the shared library packaging policy.
* Wed Mar 20 2013 zaitor@opensuse.org
- Update to version 2.0.16:
  + Allow the Callback::lambda() factory function to be passed
    callable objects which are lvalues (callback.h).
  + Deal with document format breakage from doxygen markup changes.
  + Update documentation.
- Changes from version 2.0.15:
  + Make global static Notifier objects safe.
  + Avoid use of GObject-style casts in inline functions and
    function templates to enable user code defining
    G_DISABLE_CAST_CHECKS to be more easily compliant with the
    one-definition rule.
  + Remove some other unnecessary GObject-style casts, where the
    object concerned is created in the function where the cast is
    carried out.
  + Avoid deprecation warning for g_type_init() with glib>=2.36
  + Provide a --without-gtk configuration option, to enable the
    library to be compiled without classes which require GTK+
    support.
  + Update documentation.
* Mon Jan 14 2013 zaitor@opensuse.org
- Update to version 2.0.14:
  + Add a Cgu::Thread::Taskmanager::change_max_threads() method
  + Generalise the Cgu::Thread::Future and Cgu::Thread::TaskManager
    methods which take a function object so as to take the object
    via a template parameter rather than via a std::function
    object, and deduce return types automatically in the
    Cgu::Thread::Taskmanger::make_task_*() and
    Cgu::Thread::make_future() methods.
  + Make efficiency improvement when executing 'when' and 'fail'
    callback in a Cgu::Thread::TaskManager object and callback in a
    Cgu::Thread::Future object.
  + Add test to Cgu::Thread::TaskManager unit tests
  + Update documentation.
* Sun Dec  2 2012 dimstar@opensuse.org
- Update to version 2.0.13:
  + Provide TaskManager::make_task_result(),
    TaskManager::make_task_when(),
    TaskManager::make_task_when_full() and
    TaskManager::make_task_compose() convenience wrapper methods
    for target functions which return a value, for easier task
    composition.
  + Don't use static member function of local struct for callback
    function.
  + Make explicit move operations where relevant.
  + Update unit tests.
  + Update documentation.
* Sun Oct  7 2012 zaitor@opensuse.org
- Update to version 2.0.12:
  + Provide a Cgu::Thread::TaskManager thread pool class for
    multiple task to thread mapping and task_manager.
  + Suppress deprecated warnings for g_thread_init(),
    GStaticRecMutex and GtkTable with glib-2.34/gtk+-3.6.
  + Replace the sourceforge SVN repository with a sourceforge
    git respository.
  + Update documentation.
* Sun Sep 16 2012 dimstar@opensuse.org
- Update to version 2.0.11:
  + Add move_pop() methods to AsyncQueue and AsyncQueueDispatch
    classes, and move_pop_dispatch() and move_pop_timed_dispatch()
    methods to the AsyncQueueDispatch class.
  + Add move_get() method to AsyncResult class.
  + Add move_get() method to Thread::Future class.
  + Make AsyncResult::get() truly cancellation safe
  + Carry out locking in the AsyncResult destructor.
  + Make Thread::Future::get() cancellation safe.
  + Make sure Thread::Future::get() does not hang if a call to
    Thread::Future::run() by another thread threw.
  + Merge c++11 branch with trunk in SVN.
  + Update documentation.
* Sun Jul 22 2012 zaitor@opensuse.org
- Update to version 2.0.10:
  + Enable Callback objects created with Cgu::Callback::lambda() to
    be constructed for mutable lambda objects.
  + Update build system to libtool-2.4.2, automake-1.12.1 and
    autoconf-2.69.
  + Update doxygen configuration file for doxygen-1.8.1.1.
  + Update unit test.
  + Update documentation.
* Mon Jun 11 2012 dimstar@opensuse.org
- Update to version 2.0.9:
  + Provide a new Callback::lambda() factory function for more
    convenient construction of callback objects from C++11 lambda
    expressions.
  + Provide for unblocking of threads waiting on an
    AsyncQueueDispatch object where it acquires items from a move
    assignment.
  + Update unit tests.
  + Update documentation.
* Sun May 20 2012 dimstar@opensuse.org
- Update to version 2.0.8:
  + Provide new Cgu::AsyncResult class.
  + Provide copy constructors, assignment operators and swap
    methodsfor Cgu::AsyncQueue and Cgu::AsyncQueueDispatch classes.
  + Provide size() method for Cgu::AsyncQueue and
    Cgu::AsyncQueueDispatch classes.
  + Add default constructor for Cgu::Thread::JoinableHandle class.
  + Add gcc-4.7 to the list of compilers requiring use of the
  - std=c++0x flag.
  + Update other documentation.
* Thu Apr 12 2012 malcolmlewis@opensuse.org
- Update to version 2.0.7:
  + Fix compilation with gcc-4.7.
  + Provide unit test for Future asynchronous functions.
  + Avoid types deprecated in gtk+-3.4 in the Application unit
    test, when compiled against gtk+-3.4.
  + Update FSF address in relevant copyright notices.
  + Update documentation.
- Drop c++-gtk-utils-fix-incorrect-fsf-address.patch: fixed
  upstream.
- Drop c++-gtk-utils-gcc47.patch: fixed upstream.
* Sat Mar 17 2012 dimstar@opensuse.org
- Add c++-gtk-utils-gcc47.patch: Fix build with gcc 4.7.
* Sat Feb  4 2012 malcolmlewis@opensuse.org
- Initial build.
- Add c++-gtk-utils-fix-incorrect-fsf-address.patch: Fix incorrect
  FSF address (sf#3484267).
